/*
**
** Test backprop.c performance by displaying MCUPS.
** $Header: /da/bilmes/phipac/backprop/RCS/backprop_perf.c,v 1.2 1995/12/14 01:52:04 bilmes Exp $
*/


#include <stdio.h>
#include <stdlib.h>

#include <IPM_timer.h>

#define NINPUTS 9*18
#define NHIDDEN 512
#define NOUTPUTS 61

#define DEFNUMPATS 32

extern float
bunched_mlp2_backprop (
       const int Ni,     /* Number input units */
       const int Nh,     /* Number hidden units */
       const int No,     /* Number output units */
       const int Np,     /* number patterns in a bunch (i.e., bunch size). */
       float *who, /* hidden to output weight matrix NoX(Nh+1) */
       float *wih, /* input to hidden weight matrix NhX(Ni+1) */
       /* for ipats and tpats, there are Np patterns (the bunch size)
	  and each one is a row in the corresponding matrix. */
       float *ipats, /* input patterns  NpX(Ni+1) */
       float *tpats, /* target patterns NpXNo */
       /* preallocated from the outside */
       float *hpats,  /* hidden layer NpX(Nh+1) */
       float *dhids,  /* delta hiddens NpXNh */
       float *opats,  /* output layer NpXNo */
       float *douts,  /* delta outputs NpXNo  */
       float alpha );  /* learning rate */

void
finit(float *p,int size)
{
  double drand48();
  float *endp = &p[size];
  while (p != endp)
    *p++ = drand48();
}


float *
fmalloc(int size)
{
  float *res;
  res = (float*)malloc(sizeof(float)*size);
  if (res == NULL) {
    fprintf(stderr,"Error: Out of memory\n");
    exit(-1);
  }
  return res;
}

void
usage(char *name)
{
  fprintf(stderr,"Usage:\n");
  fprintf(stderr,"  %s num_inputs num_hidden num_outputs bunch_size iters tests\n",
	  name);
  exit(0);
}

int
main(int argc, char *argv[])
{

  int Ni,Nh,No,Np;
  float *who; /* hidden to output weight matrix NoX(Nh+1) */
  float *wih; /* input to hidden weight matrix NhX(Ni+1) */
  float *ipats; /* input patterns  NpX(Ni+1) */
  float *tpats; /* target patterns NoXNp */
  float *hpats;  /* hidden layer */
  float *opats;  /* output layer */
  float *douts;  /* delta outputs */
  float *dhids;  /* delta hiddens */
  float alpha=0.1;
  int iters,tests,i,j;
  IPM_timer itimer;

  if (argc < 7)
    usage(argv[0]);

  Ni = atoi(argv[1]);
  Nh = atoi(argv[2]);
  No = atoi(argv[3]);
  Np = atoi(argv[4]);
  iters = atoi(argv[5]);
  tests = atoi(argv[6]);

  who = fmalloc(No*(Nh+1));    finit(who,No*(Nh+1));
  wih = fmalloc(Nh*(Ni+1));    finit(wih,Nh*(Ni+1));
  ipats = fmalloc(Np*(Ni+1));  finit(ipats,Np*(Ni+1));
  tpats = fmalloc(Np*No);  finit(tpats,Np*No);

  hpats = fmalloc(Np*(Nh+1));  finit(hpats,Np*(Nh+1));
  opats = fmalloc(Np*No);  finit(opats,Np*No);
  douts = fmalloc(Np*No);  finit(douts,Np*No);
  dhids = fmalloc(Np*Nh);  finit(dhids,Np*Nh);


  for (i=0;i<tests;i++) {
    double time;
    IPM_timer_clear(&itimer);
    IPM_timer_start(&itimer);
    for (j=0;j<iters;j++) {
      bunched_mlp2_backprop(Ni,Nh,No,Np,
	       who,wih,ipats,tpats,
	       hpats,
	       dhids,
	       opats,
	       douts,
	       alpha);
    }
    IPM_timer_stop(&itimer);
    time = IPM_timer_read(&itimer);
    printf("%f ",(iters*Np*(Nh+1)*((Ni+1)+ No))/(1e6*time));
    fflush(stdout);
  }
  printf("\n");
  exit(0);
}



